/*
 * MoabTests.m
 *
 * Copyright (c) 2007 Landon Fuller <landonf@opendarwin.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of Landon Fuller nor the names of any contributors
 *    may be used to endorse or promote products derived from this
 *    software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */


#import "MoabTests.h"
#include "Moab.h"

#ifdef DEBUG_BUILD
static const char imagePath[] = "build/Debug/Tests.octest/Contents/MacOS/Executable";
#else
static const char imagePath[] = "build/Release/Tests.octest/Contents/MacOS/Executable";
#endif


@implementation MoabTests

- (void) test_MOABCreateBundleForImagePathDepth {
    CFDictionaryRef bundleInfoDict;
    CFStringRef cfExecutable;
    CFBundleRef bundle;

    bundle = MOABCreateBundleForImagePathDepth(imagePath, kMoabImageBundleDepth);
    STAssertTrue(bundle != NULL, @"NULL bundle returned");
    bundleInfoDict = CFBundleGetInfoDictionary(bundle);
    cfExecutable = CFDictionaryGetValue(bundleInfoDict, CFSTR("CFBundleExecutable"));
    STAssertTrue(cfExecutable != NULL, @"NULL CFBundleExecutable returned");
    STAssertTrue(CFEqual(cfExecutable, CFSTR("Tests")), @"Unexpected bundle NSExecutable: %@", cfExecutable);
    CFRelease(bundle);
}

- (void) test_MOABMatchVersionForImageBundle {
    STAssertTrue(MOABMatchVersionForImageBundle(imagePath, kMoabImageBundleDepth, CFSTR("1.0")), @"Expected to match on version 1.0");
}

@end
