/*
 * PreferencePane.m
 *
 * Copyright (c) 2007 Landon Fuller <landonf@opendarwin.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of Landon Fuller nor the names of any contributors
 *    may be used to endorse or promote products derived from this
 *    software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include "PreferencePane.h"
#include "MoabShared.h"

#include <ApplicationEnhancer/ApplicationEnhancer.h>
#include <ApplicationEnhancer/APETools.h>


/* Main menu NIB */
static NSString *mainMenu = @"MainMenu";


@implementation OrgBikemonkeyLandonMoabPreferencePane

/*
 * Initialization
 */

/* Initialize the preference pane with the enclosing ape bundle, and our own preference bundle */
- initWithAPE:(NSBundle *)apeBundle andBundle:(NSBundle *)prefPaneBundle {
    [self init];
    if (!self)
        return self;

    [NSBundle loadNibNamed: mainMenu owner: self];

    return self;
}


/*
 * UI Management
 */

/* Called once our NIB is loaded */
- (void) awakeFromNib {
    Boolean prefExists;
    Boolean warnDMG;

    /* Match GUI options to our preferences */
    warnDMG = APEToolsGetApplicationBoolean(kMoabBundleId, kAPEToolsGlobalApplication, kMoabDMGWarnPref, &prefExists);

    /* Defaults to On */
    if (!prefExists) {
        warnDMG = true;
        APEToolsSetApplicationBoolean(kMoabBundleId, kAPEToolsGlobalApplication, kMoabDMGWarnPref, warnDMG);
    }

    if (warnDMG) {
        [warnDMGButton setState: NSOnState];
    } else {
        [warnDMGButton setState: NSOffState];
    }
}

/* Warn DMG Button Action */
- (IBAction) warnDMGButtonAction: (id) sender {
    Boolean warnDMG;

    if ([sender state] == NSOnState) {
        warnDMG = true;
    } else {
        warnDMG = false;
    }

    /* Set the new preference */
    APEToolsSetApplicationBoolean(kMoabBundleId, kAPEToolsGlobalApplication, kMoabDMGWarnPref, warnDMG);

    /* Announce a preference sync to all existing loaded modules */
    APEMessageBroadcast(kMoabBundleId, kMoabPreferencesChanged, NULL);
}


/*
 * APE Entry Points
 */

/* Return the id of the class that implements the APE Manager preference protocol */
+ (id) prefPaneHandlerForAPE:(NSBundle *) apeBundle andBundle:(NSBundle *)prefPaneBundle {
    return [[self alloc] initWithAPE: apeBundle andBundle: prefPaneBundle];
}


/* Called when the preference pane is about to be loaded */
- (void) willLoad {
    // Do nothing
}

/* Called when the preference pane is about to be unloaded */
- (void) willUnload {
    // Do nothing
}

/* Returns an NSView to be loaded into the APE manager preference area */
- (NSView *) mainView {
    return [window contentView];
}


@end
